#include <iostream>
#include <cstdio>
#include <cmath>
#include <vector>
#include <algorithm>
using namespace std;

const double Pi = 3.141592653;

const long MAXN = 1001;

struct event{
  double time;
  bool u;
  
  event(double t, bool x) : time(t), u(x) {}
  
  friend bool operator<(const event &a, const event &b) {
    if (fabs(a.time - b.time) < 1e-9) 
      return a.u && !b.u;
    else 
      return a.time < b.time;
  }
};

double cx, cy;

int n;
double mx[MAXN], my[MAXN];
double px[MAXN], py[MAXN];
double speed[MAXN];

int K;

inline double dist(double x1, double y1, double x2, double y2) {
  return sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2));
}

bool intersection(double px, double py, double qx, double qy, double r, double &x1, double &y1, double &x2, double &y2) {
  double A = qy - py, B = px - qx, C =  px * qy - py * qx;
  double A1 = -B, B1 = A, C1 = A1 * cx + B1 * cy;
  
  double d = A * B1 - A1 * B;
  double dx = C * B1 - C1 * B;
  double dy = A * C1 - A1 * C;
  
  double xpr = dx / d, ypr = dy / d;
  
  double z = dist(xpr, ypr, cx, cy);
  if (z > r + 1e-9) 
    return false;
  else {
    double theta = acos(z / r);
    
    double tng = atan2(ypr - cy, xpr - cx);
    if (tng < 0) 
      tng += 2 * Pi;
    
    double u1 = tng - theta;
    if (u1 < 0) 
      u1 += 2 * Pi;
    
    double u2 = tng + theta;
    if (u2 >= 2 * Pi)
      u2 -= 2 * Pi;
    
    x1 = cx + r * cos(u1), y1 = cy + r * sin(u1);
    x2 = cx + r * cos(u2), y2 = cy + r * sin(u2);
    
    return true;
  }
}


bool solve(double r){
  vector<event> v;
  for (int i = 0; i < n; i++){
    double x1, y1, x2, y2;
    
    if (intersection(mx[i], my[i], px[i], py[i], r, x1, y1, x2, y2)) {	
      if (dist(mx[i], my[i], cx, cy) < r + 1e-9) {
	v.push_back(event(0.0, true));
	if (px[i] != mx[i]) {
	  if ((x1 - mx[i]) / (px[i] - mx[i]) > 0)
	    v.push_back(event(dist(x1, y1, mx[i], my[i]) / speed[i], false));
	  else
	    v.push_back(event(dist(x2, y2, mx[i], my[i]) / speed[i], false));
	} else {
	  if ((y1 - my[i]) / (py[i] - my[i]) > 0)
	    v.push_back(event(dist(x1, y1, mx[i], my[i]) / speed[i], false));
	  else
	    v.push_back(event(dist(x2, y2, mx[i], my[i]) / speed[i], false));
	}
      } else if ((px[i] != mx[i] && (x1 - mx[i]) / (px[i] - mx[i]) > 0) || (py[i] != my[i] && (y1 - my[i]) / (py[i] - my[i]) > 0)) {
	double t1 = dist(x1, y1, mx[i], my[i]) / speed[i];
	double t2 = dist(x2, y2, mx[i], my[i]) / speed[i];
	if (t1 > t2) 
	  swap(t1, t2);
	v.push_back(event(t1, true));
	v.push_back(event(t2, false));
      }
    }
  }
  
  sort(v.begin(), v.end());
  
  int cnt = 0;
  for (int i = 0; i < v.size(); i++){
    if (v[i].u) 
      cnt++;
    else 
      cnt--;
		
    if (cnt == K)
      return true;
  }
  
  return false;
}


void init(){
  scanf("%lf %lf", &cx, &cy);
  scanf("%d %d", &n, &K);
  for (int i = 0; i < n; i++){
    scanf("%lf %lf %lf %lf", &mx[i], &my[i], &px[i], &py[i]);
    speed[i] = dist(mx[i], my[i], px[i], py[i]);
  }
}

int main(){
  init();
  
  double l = 1e-5, r = 15000.0;
  for (int tt = 0; tt < 100; ++tt){
    double m = (l + r) / 2.0;
    if (solve(m))
      r = m;
    else
      l = m;
  }
  
  printf("%.5lf\n", l);
  
  return 0;
}
