// This is forward search with the minimum remaining
// values heuristic.  It's also plain old backtracking
// search with basic propagation (they're the same for
// this problem).

// Note to reader: the "dual" problem where the variables are the positions
// of mines neighboring a cell turns out to work better if you're trying
// to implement a really nice solver.  
// cases.

#include <iostream>
#include <vector>
#include <queue>
#include <algorithm>
#include <cassert>

using namespace std;

typedef vector<int> VI;
typedef vector<VI> VVI;

int sols = 0;

#define MINE 2
#define NOMINE 1

int X,Y;
struct cell
{
  signed char num;  // -1: unknown.
  unsigned char domain;  // 1: empty. 2: mine. 3: unknown.
  signed char adj_mines;
  signed char adj_empty;
  signed char assigned;  // true if assigned.
};

typedef vector<cell> VC;
typedef vector<VC> VVC;

VVC board;

int dx[] = {-1, 0, 1, 1, 1, 0,-1,-1};
int dy[] = { 1, 1, 1, 0,-1,-1,-1, 0};

struct Invalid {};

void prop(VVC &board, int x, int y)
{
  cell &c = board[x][y];
  if (c.num == -1)
    return;  // No constraint.
#ifndef BAD_SOLUTION
  assert(c.adj_mines <= c.num);
  assert(c.adj_empty <= 8-c.num);
#else
  if (c.adj_mines > c.num || c.adj_empty > 8-c.num)
    throw Invalid();
#endif
  if (c.adj_mines + c.adj_empty == 8)
    return;
  unsigned char mask = 3;
  if (c.adj_mines == c.num)
    mask &= NOMINE;
  else if (c.adj_empty == 8-c.num)
    mask &= MINE;
  if (mask == 0)
    throw Invalid();

  if (mask == 3)
    return;

  for(int i = 0; i < 8; i++)
    {
      int xx = x+dx[i];
      int yy = y+dy[i];
      if (xx >= 0 && xx < X && yy >= 0 && yy < Y && !board[xx][yy].assigned)
	{
	  board[xx][yy].domain &= mask;
	  if (board[xx][yy].domain == 0)
	    throw Invalid();
	}
    }
}

void assign(VVC &board, int x, int y)
{
  cell &c = board[x][y];
  assert(!c.assigned);
  c.assigned = true;
  for(int i = 0; i < 8; i++)
    {
      int xx = x+dx[i];
      int yy = y+dy[i];
      if (xx >= 0 && xx < X && yy >= 0 && yy < Y)
	{
	  if (c.domain == MINE)
	    board[xx][yy].adj_mines++;
	  else if (c.domain == NOMINE)
	    board[xx][yy].adj_empty++;
	  else
	    assert(false);
	}
    }
  for(int i = 0; i < 8; i++)
    {
      int xx = x+dx[i];
      int yy = y+dy[i];
      if (xx >= 0 && xx < X && yy >= 0 && yy < Y)
	  prop(board, xx, yy);
    }
}

VVI result;

void solve(VVC board)
{
  bool cont = true;
  bool allassigned;
  int nextx = -1, nexty;
  while(cont)
    {
      cont = false;
      allassigned = true;
      for(int x = 0; x < X; x++)
	for(int y = 0; y < Y; y++)
	  {
#ifndef BAD_SOLUTION
	    if ((board[x][y].domain == 1 || board[x][y].domain == 2) &&
		!board[x][y].assigned) {
	      try {
		assign(board, x, y);
	      } catch (Invalid&) {
		return;  // No solution.
	      }
	      cont = true;
	    }
#endif

	    if (!board[x][y].assigned) {
	      nextx = x; nexty = y;
	      allassigned = false;
	    }
	  }

      if (allassigned) {
        ++sols;
	for(int x = 0; x < X; x++)
	  for(int y = 0; y < Y; y++)
	    result[x][y] |= board[x][y].domain;
	return;
      }
    }

  assert(nextx != -1);
  int origdom = board[nextx][nexty].domain;
#ifndef BAD_SOLUTION
  assert(origdom == 3);
  board[nextx][nexty].domain = MINE;
  solve(board);
  board[nextx][nexty].domain = NOMINE;
  solve(board);
#else
  assert(origdom != 0 && !board[nextx][nexty].assigned);
  if (origdom & MINE) {
    VVC board2 = board;
    board2[nextx][nexty].domain = MINE;
    try {
      assign(board2, nextx, nexty);
      solve(board2);
    } catch (Invalid &) {
    }
  }
  if (origdom & NOMINE) {
    VVC board2 = board;
    board2[nextx][nexty].domain = NOMINE;
    try {
      assign(board2, nextx, nexty);
      solve(board2);
    } catch (Invalid &) {
    }
  }
#endif
}

int main()
{
  cin >> X >> Y;
  board = VVC(X, VC(Y));
  result = VVI(X, VI(Y, 0));
  for(int y = 0; y < Y; y++)
    for(int x = 0; x < X; x++)
      {
	char c;
	cin >> c;
	if (c >= '0' && c <= '8') {
	  board[x][y].num = c-'0';
	  board[x][y].domain = NOMINE;
	} else if (c == '.') {
	  board[x][y].num = -1;
	  board[x][y].domain = MINE | NOMINE;
	} else {
	  assert(false);
	}
	board[x][y].assigned = false;
	board[x][y].adj_mines = 0;
	board[x][y].adj_empty = 0;
	for(int i = 0; i < 8; i++)
	  {
	    int xx = x+dx[i];
	    int yy = y+dy[i];
	    if (!(xx >= 0 && xx < X && yy >= 0 && yy < Y))
	      board[x][y].adj_empty++;  // No mine there.
	  }
      }

  for(int x = 0; x < X; x++)
    for(int y = 0; y < Y; y++)
      {
	if (board[x][y].num != -1)
	  goto next;
	for(int i = 0; i < 8; i++)
	  {
	    int xx = x+dx[i];
	    int yy = y+dy[i];
	    if (xx >= 0 && xx < X && yy >= 0 && yy < Y)
	      if (board[xx][yy].num != -1)
		goto next;
	  }

	board[x][y].domain = NOMINE;  // Doesn't really matter.
	result[x][y] = MINE | NOMINE;
      next:
	;
      }

  solve(board);

  for(int y = 0; y < Y; y++)
    {
      for(int x = 0; x < X; x++)
	{
	  if (board[x][y].num != -1)
	    cout << (char)(board[x][y].num + '0');
	  else if (result[x][y] == MINE)
	    cout << '*';
	  else if (result[x][y] == NOMINE)
	    cout << ' ';
	  else if (result[x][y] == (MINE | NOMINE))
	    cout << '.';
	  else
	    assert(false);
	}
      cout << endl;
    }
}
